@@ -1,3 +1,5 @@
+# coding=UTF-8
+
 # Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
 # Copyright (C) 2009-2013 Kai Willadsen <kai.willadsen@gmail.com>
 #
@@ -42,6 +44,7 @@ from collections import namedtuple
 from decimal import Decimal
 
 from meld.conf import _
+from meld.misc import all_same
 from meld.settings import bind_settings, meldsettings, settings
 
 
@@ -86,10 +89,6 @@ Same, SameFiltered, DodgySame, DodgyDifferent, Different, FileError = \
 CHUNK_SIZE = 4096
 
 
-def all_same(lst):
-    return not lst or lst.count(lst[0]) == len(lst)
-
-
 def remove_blank_lines(text):
     splits = text.splitlines()
     lines = text.splitlines(True)
@@ -109,8 +108,7 @@ def _files_same(files, regexes, comparison_args):
       FileError: There was a problem reading one or more of the files
     """
 
-    # One file is the same as itself
-    if len(files) < 2:
+    if all_same(files):
         return Same
 
     files = tuple(files)
@@ -121,7 +119,7 @@ def _files_same(files, regexes, comparison_args):
     time_resolution_ns = comparison_args['time-resolution']
     ignore_blank_lines = comparison_args['ignore_blank_lines']
 
-    need_contents = regexes or ignore_blank_lines
+    need_contents = comparison_args['apply-text-filters']
 
     # If all entries are directories, they are considered to be the same
     if all([stat.S_ISDIR(s.mode) for s in stats]):
@@ -143,7 +141,7 @@ def _files_same(files, regexes, comparison_args):
         return Different
 
     # Check the cache before doing the expensive comparison
-    cache_key = (files, regexes, ignore_blank_lines)
+    cache_key = (files, need_contents, regexes, ignore_blank_lines)
     cache = _cache.get(cache_key)
     if cache and cache.stats == stats:
         return cache.result
@@ -192,6 +190,9 @@ def _files_same(files, regexes, comparison_args):
 
     if result == Different and need_contents:
         contents = ["".join(c) for c in contents]
+        # For probable text files, discard newline differences to match
+        # file comparisons.
+        contents = ["\n".join(c.splitlines()) for c in contents]
         for r in regexes:
             contents = [re.sub(r, "", c) for c in contents]
         if ignore_blank_lines:
@@ -253,9 +254,18 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         ('folder-shallow-comparison', 'shallow-comparison'),
         ('folder-time-resolution', 'time-resolution'),
         ('folder-status-filters', 'status-filters'),
+        ('folder-filter-text', 'apply-text-filters'),
         ('ignore-blank-lines', 'ignore-blank-lines'),
     )
 
+    apply_text_filters = GObject.property(
+        type=bool,
+        nick="Apply text filters",
+        blurb=(
+            "Whether text filters and other text sanitisation preferences "
+            "should be applied when comparing file contents"),
+        default=False,
+    )
     ignore_blank_lines = GObject.property(
         type=bool,
         nick="Ignore blank lines",
@@ -354,6 +364,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.current_path, self.prev_path, self.next_path = None, None, None
         self.on_treeview_focus_out_event(None, None)
         self.focus_pane = None
+        self.row_expansions = set()
 
         # One column-dict for each treeview, for changing visibility and order
         self.columns_dict = [{}, {}, {}]
@@ -420,6 +431,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.connect("notify::shallow-comparison", self.update_comparator)
         self.connect("notify::time-resolution", self.update_comparator)
         self.connect("notify::ignore-blank-lines", self.update_comparator)
+        self.connect("notify::apply-text-filters", self.update_comparator)
 
         self.state_filters = []
         for s in self.state_actions:
@@ -428,6 +440,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 action_name = self.state_actions[s][1]
                 self.actiongroup.get_action(action_name).set_active(True)
 
+        self._scan_in_progress = 0
+
     def on_style_updated(self, widget):
         style = widget.get_style_context()
 
@@ -461,6 +475,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         comparison_args = {
             'shallow-comparison': self.props.shallow_comparison,
             'time-resolution': self.props.time_resolution,
+            'apply-text-filters': self.props.apply_text_filters,
             'ignore_blank_lines': self.props.ignore_blank_lines,
         }
         self.file_compare = functools.partial(
@@ -546,7 +561,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             if f.filter is None:
                 disabled_actions.append(name)
 
-        self.filter_actiongroup = Gtk.ActionGroup("DirdiffFilterActions")
+        self.filter_actiongroup = Gtk.ActionGroup(name="DirdiffFilterActions")
         self.filter_actiongroup.add_toggle_actions(actions)
         for name in disabled_actions:
             self.filter_actiongroup.get_action(name).set_sensitive(False)
@@ -631,12 +646,12 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         for i, l in enumerate(locations):
             if not isinstance(l, unicode):
                 locations[i] = l.decode(sys.getfilesystemencoding())
-        # TODO: Support for blank folder comparisons should probably look here
-        locations = [os.path.abspath(l or ".") for l in locations]
+        locations = [os.path.abspath(l) if l else '' for l in locations]
         self.current_path = None
         self.model.clear()
         for pane, loc in enumerate(locations):
-            self.fileentry[pane].set_filename(loc)
+            if loc:
+                self.fileentry[pane].set_filename(loc)
         child = self.model.add_entries(None, locations)
         self.treeview0.grab_focus()
         self._update_item_state(child)
@@ -662,6 +677,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.model.remove(child)
             child = self.model.iter_children( it )
         self._update_item_state(it)
+        self._scan_in_progress += 1
         self.scheduler.add_task(self._search_recursively_iter(path))
 
     def _search_recursively_iter(self, rootpath):
@@ -816,16 +832,58 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             if differences:
                 expanded.add(path)
 
-        self._show_tree_wide_errors(invalid_filenames, shadowed_entries)
+        if invalid_filenames or shadowed_entries:
+            self._show_tree_wide_errors(invalid_filenames, shadowed_entries)
+        elif not expanded:
+            self._show_identical_status()
 
+        self.treeview[0].expand_to_path(Gtk.TreePath(("0",)))
         for path in sorted(expanded):
             self.treeview[0].expand_to_path(path)
         yield _("[%s] Done") % self.label_text
 
         self.scheduler.add_task(self.on_treeview_cursor_changed)
+        self._scan_in_progress -= 1
         self.treeview[0].get_selection().select_path(Gtk.TreePath.new_first())
         self._update_diffmaps()
 
+    def _show_identical_status(self):
+        primary = _("Folders have no differences")
+        identical_note = _(
+            "Contents of scanned files in folders are identical.")
+        shallow_note = _(
+            "Scanned files in folders appear identical, but contents have not "
+            "been scanned.")
+        file_filter_qualifier = _(
+            "File filters are in use, so not all files have been scanned.")
+        text_filter_qualifier = _(
+            "Text filters are in use and may be masking content differences.")
+
+        is_shallow = self.props.shallow_comparison
+        have_file_filters = any(f.active for f in self.name_filters)
+        have_text_filters = any(f.active for f in self.text_filters)
+
+        secondary = [shallow_note if is_shallow else identical_note]
+        if have_file_filters:
+            secondary.append(file_filter_qualifier)
+        if not is_shallow and have_text_filters:
+            secondary.append(text_filter_qualifier)
+        secondary = " ".join(secondary)
+
+        for pane in range(self.num_panes):
+            msgarea = self.msgarea_mgr[pane].new_from_text_and_icon(
+                Gtk.STOCK_INFO, primary, secondary)
+            button = msgarea.add_button(_("Hide"), Gtk.ResponseType.CLOSE)
+            if pane == 0:
+                button.props.label = _("Hi_de")
+
+            def clear_all(*args):
+                for p in range(self.num_panes):
+                    self.msgarea_mgr[p].clear()
+            msgarea.connect("response", clear_all)
+            msgarea.show_all()
+
+
     def _show_tree_wide_errors(self, invalid_filenames, shadowed_entries):
         header = _("Multiple errors occurred while scanning this folder")
         invalid_header = _("Files with invalid encodings found")
@@ -877,45 +935,61 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         return msgarea
 
     def copy_selected(self, direction):
-        assert direction in (-1,1)
+        assert direction in (-1, 1)
         src_pane = self._get_focused_pane()
-        if src_pane is not None:
-            dst_pane = src_pane + direction
-            assert dst_pane >= 0 and dst_pane < self.num_panes
-            paths = self._get_selected_paths(src_pane)
-            paths.reverse()
-            model = self.model
-            for path in paths: #filter(lambda x: x.name is not None, sel):
-                it = model.get_iter(path)
-                name = model.value_path(it, src_pane)
-                if name is None:
-                    continue
-                src = model.value_path(it, src_pane)
-                dst = model.value_path(it, dst_pane)
-                try:
-                    if os.path.isfile(src):
-                        dstdir = os.path.dirname( dst )
-                        if not os.path.exists( dstdir ):
-                            os.makedirs( dstdir )
-                        misc.copy2( src, dstdir )
-                        self.file_created( path, dst_pane)
-                    elif os.path.isdir(src):
-                        if os.path.exists(dst):
-                            if misc.run_dialog( _("'%s' exists.\nOverwrite?") % os.path.basename(dst),
-                                    parent = self,
-                                    buttonstype = Gtk.ButtonsType.OK_CANCEL) != Gtk.ResponseType.OK:
-                                continue
-                        misc.copytree(src, dst)
-                        self.recursively_update( path )
-                except (OSError, IOError, shutil.Error) as err:
-                    misc.error_dialog(
-                        _("Error copying file"),
-                        _("Couldn't copy %s\nto %s.\n\n%s") % (
-                            GLib.markup_escape_text(src),
-                            GLib.markup_escape_text(dst),
-                            GLib.markup_escape_text(str(err)),
+        if src_pane is None:
+            return
+
+        dst_pane = src_pane + direction
+        assert dst_pane >= 0 and dst_pane < self.num_panes
+        paths = self._get_selected_paths(src_pane)
+        paths.reverse()
+        model = self.model
+        for path in paths:  # filter(lambda x: x.name is not None, sel):
+            it = model.get_iter(path)
+            name = model.value_path(it, src_pane)
+            if name is None:
+                continue
+            src = model.value_path(it, src_pane)
+            dst = model.value_path(it, dst_pane)
+            try:
+                if os.path.isfile(src):
+                    dstdir = os.path.dirname(dst)
+                    if not os.path.exists(dstdir):
+                        os.makedirs(dstdir)
+                    misc.copy2(src, dstdir)
+                    self.file_created(path, dst_pane)
+                elif os.path.isdir(src):
+                    if os.path.exists(dst):
+                        parent_name = os.path.dirname(dst)
+                        folder_name = os.path.basename(dst)
+                        dialog_buttons = [
+                            (_("_Cancel"), Gtk.ResponseType.CANCEL),
+                            (_("_Replace"), Gtk.ResponseType.OK),
+                        ]
+                        replace = misc.modal_dialog(
+                            primary=_(u"Replace folder “%s”?") % folder_name,
+                            secondary=_(
+                                u"Another folder with the same name already "
+                                u"exists in “%s”.\n"
+                                u"If you replace the existing folder, all "
+                                u"files in it will be lost.") % parent_name,
+                            buttons=dialog_buttons,
+                            messagetype=Gtk.MessageType.WARNING,
                         )
+                        if replace != Gtk.ResponseType.OK:
+                            continue
+                    misc.copytree(src, dst)
+                    self.recursively_update(path)
+            except (OSError, IOError, shutil.Error) as err:
+                misc.error_dialog(
+                    _("Error copying file"),
+                    _("Couldn't copy %s\nto %s.\n\n%s") % (
+                        GLib.markup_escape_text(src),
+                        GLib.markup_escape_text(dst),
+                        GLib.markup_escape_text(str(err)),
                     )
+                )
 
     def delete_selected(self):
         """Delete all selected files/folders recursively.
@@ -984,6 +1058,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             if self.current_path and self.focus_pane:
                 self.focus_pane.set_cursor(self.current_path)
 
+        self.row_expansions = set()
+
     def on_treeview_selection_changed(self, selection, pane):
         if not self.treeview[pane].is_focus():
             return
@@ -998,12 +1074,16 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                     is_valid = False
                     break
 
+            busy = self._scan_in_progress > 0
+
             get_action("DirCompare").set_sensitive(True)
             get_action("Hide").set_sensitive(True)
-            get_action("DirDelete").set_sensitive(is_valid)
-            get_action("DirCopyLeft").set_sensitive(is_valid and pane > 0)
+            get_action("DirDelete").set_sensitive(
+                is_valid and not busy)
+            get_action("DirCopyLeft").set_sensitive(
+                is_valid and not busy and pane > 0)
             get_action("DirCopyRight").set_sensitive(
-                is_valid and pane + 1 < self.num_panes)
+                is_valid and not busy and pane + 1 < self.num_panes)
             if self.main_actiongroup:
                 act = self.main_actiongroup.get_action("OpenExternal")
                 act.set_sensitive(is_valid)
@@ -1101,11 +1181,17 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 view.expand_row(path, False)
 
     def on_treeview_row_expanded(self, view, it, path):
-        self._do_to_others(view, self.treeview, "expand_row", (path,0) )
+        self.row_expansions.add(str(path))
+        for row in self.model[path].iterchildren():
+            if str(row.path) in self.row_expansions:
+                view.expand_row(row.path, False)
+
+        self._do_to_others(view, self.treeview, "expand_row", (path, False))
         self._update_diffmaps()
 
     def on_treeview_row_collapsed(self, view, me, path):
-        self._do_to_others(view, self.treeview, "collapse_row", (path,) )
+        self.row_expansions.discard(str(path))
+        self._do_to_others(view, self.treeview, "collapse_row", (path,))
         self._update_diffmaps()
 
     def on_popup_deactivate_event(self, popup):
@@ -1426,6 +1512,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
     def recompute_label(self):
         root = self.model.get_iter_first()
         filenames = self.model.value_paths(root)
+        filenames = [f or _('No folder') for f in filenames]
         if self.custom_labels:
             label_options = zip(self.custom_labels, filenames)
             shortnames = [l[0] or l[1] for l in label_options]
@@ -1486,6 +1573,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         # do the update
         for path in changed_paths:
             self._update_item_state( model.get_iter(path) )
+        self._update_diffmaps()
 
     def next_diff(self, direction):
         if self.focus_pane:
