@@ -30,6 +30,7 @@ import os
 import re
 import shutil
 import subprocess
+import sys
 import tempfile
 
 from meld.conf import _, ngettext
@@ -65,6 +66,8 @@ class Vc(_vc.CachedVc):
         "U": _vc.STATE_CONFLICT,  # Unmerged
     }
 
+    file_encoding = sys.getfilesystemencoding()
+
     def __init__(self, location):
         super(Vc, self).__init__(location)
         self._tree_cache = {}
@@ -280,14 +283,16 @@ class Vc(_vc.CachedVc):
 
         # Get the status of files that are different in the "index" vs
         # the HEAD of the git repository
-        proc = _vc.popen([self.CMD, "diff-index",
-                          "--cached", "HEAD", path], cwd=self.location)
+        proc = _vc.popen(
+            [self.CMD, "diff-index", "--cached", "HEAD", "--relative", path],
+            cwd=self.location)
         entries = proc.read().split("\n")[:-1]
 
         # Get the status of files that are different in the "index" vs
         # the files on disk
-        proc = _vc.popen([self.CMD, "diff-files",
-                          "-0", path], cwd=self.location)
+        proc = _vc.popen(
+            [self.CMD, "diff-files", "-0", "--relative", path],
+            cwd=self.location)
         entries += (proc.read().split("\n")[:-1])
 
         # An unmerged file or a file that has been modified, added to
@@ -323,6 +328,20 @@ class Vc(_vc.CachedVc):
                 if e.errno != errno.EAGAIN:
                     raise
 
+        def get_real_path(name):
+            name = name.strip()
+            if os.name == 'nt':
+                # Git returns unix-style paths on Windows
+                name = os.path.normpath(name)
+
+            # Unicode file names and file names containing quotes are
+            # returned by git as quoted strings
+            if name[0] == '"':
+                name = name[1:-1].decode('string_escape')
+            name = name.decode(self.file_encoding)
+            return os.path.abspath(
+                os.path.join(self.location, name))
+
         if len(entries) == 0 and os.path.isfile(path):
             # If we're just updating a single file there's a chance that it
             # was it was previously modified, and now has been edited
@@ -330,34 +349,23 @@ class Vc(_vc.CachedVc):
             # 'entries' list, and tree_state['path'] will still contain stale
             # data.  When this corner case occurs we force tree_state['path']
             # to STATE_NORMAL.
-            path = os.path.abspath(path)
-            tree_state[path] = _vc.STATE_NORMAL
+            tree_state[get_real_path(path)] = _vc.STATE_NORMAL
         else:
-            # There are 1 or more modified files, parse their state
             for entry in entries:
                 columns = self.DIFF_RE.search(entry).groups()
-                old_mode, new_mode, statekey, name = columns
-                if os.name == 'nt':
-                    # Git returns unix-style paths on Windows
-                    name = os.path.normpath(name.strip())
-                path = os.path.join(self.root, name.strip())
-                path = os.path.abspath(path)
+                old_mode, new_mode, statekey, path = columns
                 state = self.state_map.get(statekey.strip(), _vc.STATE_NONE)
-                tree_state[path] = state
+                tree_state[get_real_path(path)] = state
                 if old_mode != new_mode:
                     msg = _("Mode changed from %s to %s" %
                             (old_mode, new_mode))
                     self._tree_meta_cache[path] = msg
 
-            for entry in ignored_entries:
-                path = os.path.join(self.location, entry.strip())
-                path = os.path.abspath(path)
-                tree_state[path] = _vc.STATE_IGNORED
+            for path in ignored_entries:
+                tree_state[get_real_path(path)] = _vc.STATE_IGNORED
 
-            for entry in unversioned_entries:
-                path = os.path.join(self.location, entry.strip())
-                path = os.path.abspath(path)
-                tree_state[path] = _vc.STATE_NONE
+            for path in unversioned_entries:
+                tree_state[get_real_path(path)] = _vc.STATE_NONE
 
     def _lookup_tree_cache(self, rootdir):
         # Get a list of all files in rootdir, as well as their status
